%-- set vehicle parameters
fprintf('setting vehicle parameters...\n')
ms = 9000;  %-- sprung-mass (kg)
mu = 1000;  %-- unsprung-mass (kg)
cs = 2000;  %-- sprung-damping (kg/s)
ks = 9000;  %-- sprung-stifness (N/m)=(kg/s/s)
ku = 20000; %-- unsprung-stifness (N/m)=(kg/s/s)
v = 10.0;  %-- speed (m/s)
dt= 0.001; %-- time increment;
T = 60;    %-- end time

%-- calculate input
fprintf('calculating vehicle input...\n')
R = load('profile-1.txt'); %-- load: R(x)
x = 0:0.20:(length(R)-1)*0.20; %-- dx=0.20(m) for R(x)
t = 0:dt:T; %-- time, dt=0.001(s)
xx= v*t; %-- vehicle position: x(t)
r = spline(x,R,xx); %-- spline interpolation: R(x) to r(t)=R(x(t))
dr= 1/dt*gradient(r); %-- numerical differential
r=r-r(1); %-- reference value setting: r(0)=0
dr=dr-dr(1); %-- reference value setting: dr(0)=0
f=c*dr+k*r; %-- right-hand-side of equation of motion of the vehicle

%-- simulate output
fprintf('vehicle vibration simulation: newmark-beta method...\n')
G = 1/2; %-- gammma coefficient for newmark-beta method
B = 1/4; %-- beta coefficient for newmark-beta method
M = [ms 0; 0 mu];
C = [cs -cs; -cs cs];
K = [ks -ks; -ks ks+ku];
F = [zeros(1,length(f)); f];
[ t, z, dz, ddz ] = newmark_beta( M,C,K,F,1/1000,zeros(2,3), G, B );

%-- visualization
fprintf('outputting the simulation results...\n')
figure(1)
subplot(3,1,1); plot(t,r,'-b')
subplot(3,1,2); plot(t,z(1,:),'-r')
subplot(3,1,3); plot(t,z(2,:),'-r')

%-- animation
figure(2);set(gcf, 'Unit', 'pixel', 'Position',[120, 280, 800, 300]);
writerObj = VideoWriter('ex_02.avi','Motion JPEG AVI'); %-- setting movie class
writerObj.FrameRate = 20; %-- flame rate (slide/sec)
open(writerObj); %-- initialization of movie object
for tt=1:100:length(t)
    plot(xx(1:tt),r(1,1:tt),'b-');hold on
    plot(xx(tt),r(1,tt),'bo');
    plot(xx(1:tt),z(1,1:tt)+0.02,'r-');
    plot(xx(tt),z(1,tt)+0.02','ro');
    plot(xx(1:tt),z(2,1:tt)+0.01,'r-');
    plot(xx(tt),z(2,tt)+0.01','ro');hold off
    xlim([0 max(xx)]);ylim([-0.004 0.030]);
    frame = getframe(figure(2)); %-- get the figure as a frame of the animation
    writeVideo(writerObj,frame); %-- add the gotten frame to the movie object
end
close(writerObj); %-- end of animation


