%-- set vehicle parameters
fprintf('setting vehicle parameters...\n')
ms  = 9000;  %-- sprung-mass (kg)
mu1 = 500;  %-- unsprung-mass (kg)
mu2 = 500;  %-- unsprung-mass (kg)
cs1 = 2000;  %-- sprung-damping (kg/s)
cs2 = 2000;  %-- sprung-damping (kg/s)
ks1 = 4500;  %-- sprung-stifness (N/m)=(kg/s/s)
ks2 = 4500;  %-- sprung-stifness (N/m)=(kg/s/s)
ku1 = 60000; %-- unsprung-stifness (N/m)=(kg/s/s)
ku2 = 60000; %-- unsprung-stifness (N/m)=(kg/s/s)
d1  = 1.75;  %-- distance from G to s1 (m)
d2  = 1.75;  %-- distance from G to s2 (m)
v   = 10.0;  %-- speed (m/s)
dt  = 0.001; %-- time increment
T   = 60;    %-- end time
x0  = 10;    %-- start position of G(m)

%-- calculate input
fprintf('calculating vehicle input...\n')
R = load('profile-1.txt'); %-- load: R(x)
x = 0:0.20:(length(R)-1)*0.20; %-- dx=0.20(m) for R(x)
t = 0:dt:T; %-- time, dt=0.001(s)
xx= [x0+v*t+d1; x0+v*t-d2]; %-- vehicle position: x(t)
r = spline(x,R,xx); %-- spline interpolation: R(x) to r(t)=R(x(t))

%-- simulate output
fprintf('vehicle vibration simulation: newmark-beta method...\n')
G = 1/2; %-- gammma coefficient for newmark-beta method
B = 1/4; %-- beta coefficient for newmark-beta method
I = d1*d2*ms/(d1+d2);
M = [d2*ms/(d1+d2) d1*ms/(d1+d2) 0   0   ;
     I/(d1+d2)     -I/(d1+d2)    0   0   ;
     0             0             mu1 0   ;
     0             0             0   mu2];
C = [cs1    cs2     -cs1    -cs2    ;
     d1*cs1 -d2*cs2 -d1*cs1 d2*cs2  ;
     -cs1   0       cs1     0       ;
     0      -cs2    0       cs2  ]  ;
K = [ks1    ks2     -ks1    -ks2    ;
     d1*ks1 -d2*ks2 -d1*ks1 d2*ks2  ;
     -ks1   0       ks1+ku1 0       ;
     0      -ks2    0       ks2+ku2];
F = [zeros(2,length(r)); [ku1 0; 0 ku2]*r];
[ t, z, dz, ddz ] = newmark_beta( M,C,K,F,1/1000,zeros(4,3), G, B );

%-- visualization
fprintf('outputting the simulation results...\n')
figure(1)
subplot(3,1,1); plot(t,r(1:2,:),'-b')
subplot(3,1,2); plot(t,z(1:2,:),'-r')
subplot(3,1,3); plot(t,z(3:4,:),'-r')

%-- animation
figure(2);set(gcf, 'Unit', 'pixel', 'Position',[120, 280, 800, 300]);
writerObj = VideoWriter('ex_03.avi','Motion JPEG AVI'); %-- setting movie class
writerObj.FrameRate = 20; %-- flame rate (slide/sec)
open(writerObj); %-- initialization of movie object
for tt=1:100:length(t)
    plot(xx(:,1:tt)',r(:,1:tt)','b-');hold on
    plot(xx(:,tt)',r(:,tt)','bo');
    plot(xx(:,1:tt)',z(1:2,1:tt)'+0.02,'r-');
    plot(xx(:,tt)',z(1:2,tt)'+0.02','ro');
    plot(xx(:,1:tt)',z(3:4,1:tt)'+0.01,'r-');
    plot(xx(:,tt)',z(3:4,tt)'+0.01','ro');hold off
    xlim([xx(1,tt)-70 xx(1,tt)+10]);ylim([-0.004 0.030]);
    frame = getframe(figure(2)); %-- get the figure as a frame of the animation
    writeVideo(writerObj,frame); %-- add the gotten frame to the movie object
end
close(writerObj); %-- end of animation


